import os, sys, stat
import getpass
import fnmatch
import subprocess
from anouman.templates.gunicorn_start import  (
    gunicorn_start,
    gunicorn_context, 
)

from anouman.templates.gunicorn_start import (
    gunicorn_start,
    gunicorn_context,
)

from anouman.templates.init_script import (
    build_init,
    init_context,
)

from anouman.templates.nginx import (
    nginx_upstart,
    nginx_context,
)

from anouman.templates.commands import commands


from anouman.exceptions import (
    MultipleSettingError,
    NoSettingsError,
    MultipleWSGIError,
    NoWSGIError,
    MultipleMANAGEError,
    NoMANAGEError,
)

from anouman.utils.find_files import (
    get_settings,
    get_wsgi,
    get_manage
)

def deploy_django_project(args):
    print "deploy_django_project"
    """
        The Very First thing we want to do is unpack the project
    """
    subprocess.call(["tar", "xvfz", args.deploy])
    
    # Set both args.domainname  AND args.django_project to the deploy basename
    args.domainname = args.deploy.split(".tar.gz")[0]
    args.django_project=args.domainname+"/src"

    # Ideally I'd like to call mkvirtualenv from virtualenv wrapper
    # But since it's derived from a soruced file it's not accessible
    # so instead we just call virtualenv ourselves and save in the
    # default ~/.virtualenvs/  directory
    VIRTUALENV = ".virtualenvs/%s"%(args.domainname)
    subprocess.call(["virtualenv", VIRTUALENV])

    #settings is the full path
    #SETTINGS is the django project path:  ex finance.settings
    [settings, SETTINGS] = get_settings(args)
    WSGI = get_wsgi(args)       # get module path to wsgi.py
    MANAGE = get_manage(args) #get abspath of manage.py

    BIN=os.path.abspath("%s/bin"%(VIRTUALENV))
    PIP="%s/pip"%(BIN)
    PYTHON="%s/python"%(BIN)
    ACTIVATE="%s/activate"%(BIN)
    DJANGO_VERSION="django%s"%(args.django_version)
    GUNICORN_START="%s/gunicorn_start"%(BIN)
    
    ## Now add a few shell commands to the activate script that will be
    ## unique to each deployed website
    commands.context['DOMAINNAME'] = args.domainname
    cmd_str = commands.get_commands(commands.context)
    with open(ACTIVATE, 'a') as f:   
        f.write(cmd_str)


    # Now we loop over the list of packages we stored during the packaging phase
    # and try to install them with pip
    pkg_success = []
    pgk_fails = []
    with open("%s/pip_packages.txt"%(args.domainname)) as f:
        PACKAGES = f.readlines()
        PACKAGES.append("gunicorn")
        for package in PACKAGES:
            try:
                subprocess.call([PIP, "install", package])
                pkg_success.append(package)
            except:
                pgk_fails.append(package)
    
    subprocess.call([PIP, "install", "-r", "%s/pip_packages.txt"%(args.domainname)])
    print "Package Installation Results"
    print "SUCCESS: %s" %(len(pkg_success))
    print "FAIL:    %s" %(len(pgk_fails))
    for f in pgk_fails:
        print "\t*\t%s"%(f)


    # Now we run the collectstatic command
    subprocess.call([PYTHON, MANAGE, "collectstatic"])


    """
        Now we build up a context to apply to gunicorn_start.template.
        default context located: anouman/templates/gunicorn_start/__init__.py

        We the update gunicorn_context with our context and save the file
        to {{domainname}}/bin/gunicorn_start
    """
    NAME=os.path.basename(args.django_project)
    DJANGODIR=os.path.abspath(args.domainname + "/" + NAME)

    context = {
        'NAME':args.domainname,
        'USER':getpass.getuser(),
        'GROUP':getpass.getuser(),
        'GUNICORN':"%s/gunicorn"%(BIN),
        'DJANGODIR':DJANGODIR,
        'DJANGO_SETTINGS_MODULE':SETTINGS,
        'DJANGO_WSGI_MODULE':WSGI,

        # Default bind is unix:/var/run/domainname.com.sock
        # Override with --bind=ip:port 
        'BIND':args.bind if args.bind else 'unix:/var/run/%s.sock' %(args.domainname),
    }

    gunicorn_context.update(context)

    with open(GUNICORN_START, 'w') as f:
        f.write(
            gunicorn_start(
                gunicorn_context
            )
        )
    # Set Permissions on the GUNICORN file
    os.chmod(
        GUNICORN_START,
        stat.S_IRWXU|stat.S_IRWXG|stat.S_IXOTH
     )


    """
        Now we create the gunicorn upstart scripts
    """
    NAME=args.domainname+".conf"
    context = {
        'GUNICORN_START':GUNICORN_START,
        'DOMAINNAME':args.domainname, 
    }

    init_context.update(context)
    with open(NAME, 'w') as f:
        f.write(build_init(init_context))

    print "We need to copy the website startup scripts to /etc/init/"
    print "This will require you to enter your sudo password now."
    os.system("sudo mv %s /etc/init/%s"%(NAME, NAME) )


    """
        nginx script for the site.
        place in:   /etc/nginx/sites-available
        link to:    /etc/nginx/sites-enabled 

        templ_file: anouman/templates/nginx/ngix_site.template
         
    """
    ## Import the users django settings file and grab the STATIC_ROOT and MEDIA_ROOT vars
    sys.path.append(os.path.dirname(settings))
    import settings
    nginx_context.update({
        'UNIXBIND':'unix:/var/run/%s.sock' %(args.domainname),
        'DOMAINNAME':args.domainname,
        'DJANGO_STATIC':settings.STATIC_ROOT,
        'DJANGO_MEDIA':settings.MEDIA_ROOT,
    })

    print "nginx_context: ", nginx_context
    NGINX_CONF='nginx.%s.conf'%(args.domainname)
    with open(NGINX_CONF, 'w') as f:
        f.write( nginx_upstart(nginx_context) )

    print "We need to copy the nginx.domainname.conf file to /etc/nginx/sites-available/"
    print "This will require sudo"
    os.system("sudo mv %s /etc/nginx/sites-available/%s" % (NGINX_CONF, NGINX_CONF) )
    os.system("sudo ln -s /etc/nginx/sites-available/%s /etc/nginx/sites-enabled/"% (NGINX_CONF))

    print "\n\n----------------------------------------------"
    print "Please add the following line(s) to your .bash_profile"
    print "source /usr/local/bin/virtualenvwrapper.sh;"
    print "workon %s"%(args.domainname)

def package_django_project(args):
    # settings is the full path
    # SETTINGS is the django projec path, exL  finance.settings
    [settings, SETTINGS] = get_settings(args)
    WSGI = get_wsgi(args)
    
    BIN=os.path.abspath("%s/bin"%(args.domainname))
    PIP="%s/pip"%(BIN)
    DJANGO_VERSION="django%s"%(args.django_version)
    GUNICORN_START="%s/gunicorn_start"%(BIN)
        
    os.makedirs(args.domainname)
    
    """
        Install packages from project environment into the new virtualenv.
        Also store the status for success/fail as we install them
    """
    pip = subprocess.check_output(['which', 'pip'])
    print "pip: ", pip
    print "Saving pacakges to: ", "%s/pip_packages.txt"%(args.domainname)
    os.system("pip freeze > %s/pip_packages.txt"%(args.domainname))


    """
        Now we copy your django project into the virtual env
    """
    print "Copying your source tree into the virtual env"
    print "args.django_project: ", args.django_project
    print "args.domainname: ", args.domainname
    subprocess.call(['cp', '-r', args.django_project, args.domainname+"/src"]) 
    subprocess.call(['tar', '-cf', args.domainname+".tar", args.domainname])
    subprocess.call(['gzip', args.domainname+".tar"])
    subprocess.call(["rm", "-rf", args.domainname])

"""
def django_project(args):
    if args.deploy:
        deploy_django_project(args)
    else:
        package_django_project(args)
"""
 
def new_project(args):
    BIN="%s/bin"%(args.domainname)
    PIP="%s/pip"%(BIN)
    GUNICORN="%s/gunicorn"%(BIN)
    GUNICORN_START="%s/gunicorn_start.py"%(BIN)
    DJANGO_VERSION="django%s"%(args.django_version)
    ADMIN='%s/django-admin.py'%(BIN)
    # Our very first step will be to create a brand new virtual environment
    subprocess.call(["virtualenv", args.domainname])
    subprocess.call([PIP, 'install', DJANGO_VERSION])
    # This section setups up the django server
    if args.gunicorn:
        subprocess.call([PIP, "install", "gunicorn"])

    
    # Next we need to install packaged.txt.  This will include things like django
    subprocess.call([ADMIN, 'startproject', args.project_name])

    print "Your project is ready for development"
    print "run source %s/activate" %(BIN)

    with open(GUNICORN_START, 'w') as f:
        f.write( gunicorn_start() )

    # Set Permissions on the GUNICORN file    
    os.chmod(GUNICORN_START, stat.S_IRWXU|stat.S_IRWXG|stat.S_IXOTH)


if __name__ == '__main__':
    setup("TestENV")



import os
from django.template import Template, Context
from django.conf import settings
settings.configure()

"""
    Render the gunicorn_start.py command file.

    gunicorn_start.py is a custom configued gunicorn script
    based on your django projec.

    
"""


gunicorn_context = {
    'NAME':'website',                           # Name of the application
    'DJANGODIR':'./',                           # Django project director
    'USER':'username',                          # user to run as
    'GROUP':'username',                         # group to run as
    'NUM_WORKERS':3,                            # Number of Gunicorn threads.  2(proc)+1
    'VIRTUALENVDIR':'./virtualenv',             # Directory containing your projects virtual env
    'DJANGO_SETTINGS_MODULE':'website.settings',# the django settings file
    'DJANGO_WSGI_MODULE':'website.wsgi',        # your project wsgi module
    'GUNICORN':'virtualenv/bin/gunicorn',       # location of gunicorn.py

    #   You can bind your server to a unix socket or to a port
    #   But you should only bind it to one or the other
    'BIND':False,                      # ex:  10.0.2.13:80
    'SOCKFILE':False,                   # ex:  /var/run/gunicorn.sock
}


template_dir = os.path.dirname(__file__)
def gunicorn_start(context=gunicorn_context):
    with open('%s/gunicorn_start.template'%(template_dir), 'r') as f:
        t = Template( f.read() )
        c = Context( context )
        return t.render(c)



import os
from django.template import Template, Context

"""
    Build the nginx upstart command that will run the site
"""


nginx_context = {
    'UNIXBIND':'',
    'DOMAINNAME':'',
    'DJANGO_STATIC':'',
    'DJANGO_MEDIA':'',
}


template_dir = os.path.dirname(__file__)
def nginx_upstart(context=nginx_context):
    with open('%s/ngix_site.template'%(template_dir), 'r') as f:
        t = Template( f.read() )
        c = Context( context )
        return t.render(c)



#!/usr/bin/env python
from django.template import Template, Context
import os

commands = """
#This section defined commands specified by Anouman

NGINX=/etc/init.d/nginx
DOMAINNAME={{DOMAINNAME}}

function site {
        if [ $1 == 'status' ];
        then
                sudo $NGINX status
                sudo status $DOMAINNAME
        fi

        if [ $1 == 'stop' ];
        then
                sudo $NGINX stop
                sudo stop $DOMAINNAME
        fi

        if [ $1 == 'start' ];
        then
                sudo $NGINX start
                sudo start $DOMAINNAME
        fi

        if [ $1 == 'reload' ];
        then
                sudo nginx -s reload
        fi
}"""

context = {
    'DOMAINNAME':'',
}


def get_commands(context=context):
    t = Template( commands )
    c = Context( context )
    return t.render(c)






import os
import fnmatch

def find_file(root_dir, pattern):
    """
        This is a generic recursive grep function
    """
    matches = []
    for root, dirnames, filenames in os.walk(root_dir):
        for filename in fnmatch.filter(filenames, pattern):
            matches.append(os.path.join(root, filename))
    return matches

def get_settings(args):
    """
        This function checks for the existence of the settings file.

        If the --settings options is specified then we simply
        check to see if the file exists and if so return the name of the
        file.

        If no --settings option was given then we look for settings.py
        recursivly in the directory defined by --django-project.
          -- If multiple settings.py are found we throw an exception.
          -- If not settings.py is found we throw an exception.
          -- If one settings.py is found we return it's path.
    """
    settings_path=''
    if not args.settings:
        matches = find_file(args.django_project, 'settings.py')
        if len(matches) > 1:
            raise MultipleSettingError(matches)
        elif len(matches) == 0:
            raise NoSettingsError()
        settings_path= matches[0]
    elif not os.path.isfile(args.settings):
        raise NoSettingsError() 
    else:
        settings_path = args.settings
   
    # We now have the path to the settings.py modules, but we need
    # to convert it to python module format:  settings.wsgi
    tmp = settings_path.split(args.django_project)
    if len(tmp) == 2:
        return [
            os.path.abspath(settings_path), 
            tmp[1][1:].replace("/", ".").replace(".py", "")
        ]
    else:
        raise NoSettingsError( str(tmp) )
    
def get_wsgi(args):
    """
        This function checks for the existence of the wsgi.py file.

        If the --wsgi options is specified then we simply
        check to see if the file exists and if so return the name of the
        file.

        If no --wsgi option was given then we look for wsgi.py
        recursivly in the directory defined by --django-project.
          -- If multiple wsgi.py are found we throw an exception.
          -- If not wsgi.py is found we throw an exception.
          -- If one wsgi.py is found we return it's path.
    """
    wsgi_path = ''
    if not args.wsgi:
        matches = find_file(args.django_project, 'wsgi.py')
        if len(matches) > 1:
            raise MultipleWSGIError(matches)
        if len(matches) == 0:
            raise NoWSGIError()
        wsgi_path=os.path.abspath( matches[0] )
    elif not os.path.isfile(args.wsgi):
        raise NoWSGIError() 
    else:
        wsgi_path = os.path.abspath( args.wsgi )

    # We now have the path to the wsgi.py modules, but we need
    # to convert it to python module format:  website.wsgi
    tmp = wsgi_path.split( os.path.abspath(args.django_project) )
    if len(tmp) == 2:
        return tmp[1][1:].replace("/", ".").replace(".py", "")
    else:
        raise NoWSGIError( str(tmp) )

def get_manage(args):
    """
        This function checks for the existence of the manage.py file.

        If the --manage options is specified then we simply
        check to see if the file exists and if so return the name of the
        file.

        If no --manage option was given then we look for manage.py
        recursivly in the directory defined by --django-project.
          -- If multiple manage.py are found we throw an exception.
          -- If not manage.py is found we throw an exception.
          -- If one manage.py is found we return it's path.
    """
    manage_path = ''
    if not args.manage:
        matches = find_file(args.django_project, 'manage.py')
        if len(matches) > 1:
            raise MultipleMANAGEError(matches)
        if len(matches) == 0:
            raise NoMANAGEError()
        manage_path = matches[0]
    elif not os.path.isfile(args.manage):
        raise NoMANAGEError()
    else:
        manage_path = os.path.abspath( args.manage )

    return os.path.abspath( manage_path )




import os
from django.template import Template, Context
#from django.conf import settings
#settings.configure()

"""
    Render the webiste upstart command file to be placed in /etc/init/
    in order to start the site on boot up.
"""


init_context = {
    'GUNICORN_START':'',    # The abspath to the gunicorn_start script
    'DOMAINNAME':'',        # The domain name
}


template_dir = os.path.dirname(__file__)
def build_init(context=init_context):
    with open('%s/website.conf'%(template_dir), 'r') as f:
        t = Template( f.read() )
        c = Context( context )
        return t.render(c)
